package main.java.me.dm7.barcodescanner.core;

import android.content.Context;
import android.graphics.Color;
import android.graphics.Rect;
import android.hardware.Camera;
import android.util.AttributeSet;
import android.view.Gravity;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.RelativeLayout;


import android.content.SharedPreferences;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.preference.PreferenceManager;

import main.java.me.dm7.barcodescanner.zxing.ZXingScannerView;


import anywheresoftware.b4a.BA;

public abstract class BarcodeScannerView extends FrameLayout implements Camera.PreviewCallback, SensorEventListener  {
    private Camera mCamera;
    private CameraPreview mPreview;
    private IViewFinder mViewFinderView;
    private Rect mFramingRectInPreview;
    private CameraHandlerThread mCameraHandlerThread;
    private Boolean mFlashState;
    private boolean mAutofocusState = true;

    public BarcodeScannerView(Context context) {
        super(context);
    }

    public BarcodeScannerView(Context context, AttributeSet attributeSet) {
        super(context, attributeSet);
    }

    public final void setupLayout(Camera camera) {
        removeAllViews();

        mPreview = new CameraPreview(getContext(), camera, this);
        RelativeLayout relativeLayout = new RelativeLayout(getContext());
        relativeLayout.setGravity(Gravity.CENTER);
        relativeLayout.setBackgroundColor(Color.BLACK);
        relativeLayout.addView(mPreview);
        addView(relativeLayout);

        mViewFinderView = createViewFinderView(getContext());
        if (mViewFinderView instanceof View) {
            addView((View) mViewFinderView);
        } else {
            throw new IllegalArgumentException("IViewFinder object returned by " +
                    "'createViewFinderView()' should be instance of android.view.View");
        }
    }

    /**
     * <p>Method that creates view that represents visual appearance of a barcode scanner</p>
     * <p>Override it to provide your own view for visual appearance of a barcode scanner</p>
     *
     * @param context {@link Context}
     * @return {@link android.view.View} that implements {@link ViewFinderView}
     */
    protected IViewFinder createViewFinderView(Context context) {
        return new ViewFinderView(context);
    }

    public void startCamera(int cameraId) {
        if(mCameraHandlerThread == null) {
            mCameraHandlerThread = new CameraHandlerThread(this);
        }
        mCameraHandlerThread.startCamera(cameraId);
    }

    public void setupCameraPreview(Camera camera) {
        mCamera = camera;
        if(mCamera != null) {
            setupLayout(mCamera);
            mViewFinderView.setupViewFinder();

            if(mFlashState != null) {
                setFlash(mFlashState);
            }
            setAutoFocus(mAutofocusState);
        }
    }

    public void startCamera() {
        startCamera(-1);
    }

    public void stopCamera() {
        if(mCamera != null) {
            mPreview.stopCameraPreview();
            mPreview.setCamera(null, null);
            mCamera.release();
            mCamera = null;
        }
        if(mCameraHandlerThread != null) {
            mCameraHandlerThread.quit();
            mCameraHandlerThread = null;
        }
    }

    public void stopCameraPreview() {
        if(mPreview != null) {
            mPreview.stopCameraPreview();
        }
    }

    protected void resumeCameraPreview() {
        if(mPreview != null) {
            mPreview.showCameraPreview();
        }
    }

    public synchronized Rect getFramingRectInPreview(int previewWidth, int previewHeight) {
        if (mFramingRectInPreview == null) {
            Rect framingRect = mViewFinderView.getFramingRect();
            int viewFinderViewWidth = mViewFinderView.getWidth();
            int viewFinderViewHeight = mViewFinderView.getHeight();
            if (framingRect == null || viewFinderViewWidth == 0 || viewFinderViewHeight == 0) {
                return null;
            }

            Rect rect = new Rect(framingRect);
            rect.left = rect.left * previewWidth / viewFinderViewWidth;
            rect.right = rect.right * previewWidth / viewFinderViewWidth;
            rect.top = rect.top * previewHeight / viewFinderViewHeight;
            rect.bottom = rect.bottom * previewHeight / viewFinderViewHeight;

            mFramingRectInPreview = rect;
        }
        return mFramingRectInPreview;
    }

	
	

    public void setFlash(boolean flag) {
        mFlashState = flag;
        if(mCamera != null && CameraUtils.isFlashSupported(mCamera)) {

            Camera.Parameters parameters = mCamera.getParameters();
            if(flag) {
                if(parameters.getFlashMode().equals(Camera.Parameters.FLASH_MODE_TORCH)) {
                    return;
                }
                parameters.setFlashMode(Camera.Parameters.FLASH_MODE_TORCH);
            } else {
                if(parameters.getFlashMode().equals(Camera.Parameters.FLASH_MODE_OFF)) {
                    return;
                }
                parameters.setFlashMode(Camera.Parameters.FLASH_MODE_OFF);
            }
            mCamera.setParameters(parameters);
        }
    }

	
    public boolean getFlash() {
        if(mCamera != null && CameraUtils.isFlashSupported(mCamera)) {
            Camera.Parameters parameters = mCamera.getParameters();
            if(parameters.getFlashMode().equals(Camera.Parameters.FLASH_MODE_TORCH)) {
                return true;
            } else {
                return false;
            }
        }
        return false;
    }

    public void toggleFlash() {
        if(mCamera != null && CameraUtils.isFlashSupported(mCamera)) {
            Camera.Parameters parameters = mCamera.getParameters();
            if(parameters.getFlashMode().equals(Camera.Parameters.FLASH_MODE_TORCH)) {
                parameters.setFlashMode(Camera.Parameters.FLASH_MODE_OFF);
            } else {
                parameters.setFlashMode(Camera.Parameters.FLASH_MODE_TORCH);
            }
	
            mCamera.setParameters(parameters);
        }
    }

    public void setAutoFocus(boolean state) {
        mAutofocusState = state;
        if(mPreview != null) {
            mPreview.setAutoFocus(state);
        }
    }
	
	
	
	public void zoomIn(int zoomValue) {
	  if (mCamera != null && mCamera.getParameters().isZoomSupported()) {
		Camera.Parameters parameters = mCamera.getParameters();
		BA.Log("Zoom = " + parameters.getZoom());
		BA.Log("Max Zoom = " + parameters.getMaxZoom());
		if (parameters.getZoom() > parameters.getMaxZoom()) {
		  return;
		}
		parameters.setZoom(parameters.getZoom() + zoomValue);
		mCamera.setParameters(parameters);
	  }
	}
	

	public int getCurrentZoomValue() {
	  int currentzoomvalue = 0;
	  if (mCamera != null && mCamera.getParameters().isZoomSupported()) {
		Camera.Parameters parameters = mCamera.getParameters();
		currentzoomvalue = parameters.getZoom();
	  }
	  return currentzoomvalue;
	}	
	
	public int getMaxZoomValue() {
		int maxzoomvalue = 0;
		if (mCamera != null && mCamera.getParameters().isZoomSupported()) {
			Camera.Parameters parameters = mCamera.getParameters();
			maxzoomvalue = parameters.getMaxZoom();
		}
		return maxzoomvalue;
	}
	
	
		

  private float TOO_DARK_LUX = 45.0F;
  private float BRIGHT_ENOUGH_LUX = 450.0F;
  //private final Context mcontext;
  private Sensor lightSensor;
  
  
  public void startAmbientSensor() {
      SensorManager sensorManager = (SensorManager)BA.applicationContext.getSystemService("sensor");
      lightSensor = sensorManager.getDefaultSensor(5);
      if (lightSensor != null) {
        sensorManager.registerListener(this, lightSensor, 3);
      }

  }
  
  public void stopAmbientSensor() {
    if (lightSensor != null)
    {
      SensorManager sensorManager = (SensorManager)BA.applicationContext.getSystemService("sensor");
      sensorManager.unregisterListener(this);
      lightSensor = null;
    }
  }
  
  public void onSensorChanged(SensorEvent sensorEvent) {

    float ambientLightLux = sensorEvent.values[0];
	AmbientLightLux = ambientLightLux;
    invokeAmbientLightChanged();
/*     if (AutoTorchMode == true) {
      if (mCamera != null) {
        if (ambientLightLux <= TOO_DARK_LUX) {
          flashOn();
        } else if (ambientLightLux >= BRIGHT_ENOUGH_LUX) {
          flashOff();
        }
      }
    } */
 }


 
  public void onAccuracyChanged(Sensor sensor, int accuracy) {}	
	
	

/*     private void flashOn() {
        if(mCamera != null && CameraUtils.isFlashSupported(mCamera)) {
            Camera.Parameters parameters = mCamera.getParameters();
            parameters.setFlashMode(Camera.Parameters.FLASH_MODE_TORCH);	
            mCamera.setParameters(parameters);
        }
    }	
	

    private void flashOff() {
        if(mCamera != null && CameraUtils.isFlashSupported(mCamera)) {
            Camera.Parameters parameters = mCamera.getParameters();
            parameters.setFlashMode(Camera.Parameters.FLASH_MODE_OFF);	
            mCamera.setParameters(parameters);
        }
    }	 */
	
	
	private float AmbientLightLux = 0.0f;
    public float getAmbientLightLux() {
	    return AmbientLightLux;
    }
	
	
    /**
     * Switch ON the torch if the Lux value is lower than this
     */	
    public void setLowerLuxLimit(float LowerLuxLimit) {
		TOO_DARK_LUX = LowerLuxLimit;
	}
	
    /**
     * Switch OFF the torch if the Lux value is lower than this
     */		
    public void setUpperLuxLimit(float UpperLuxLimit) {
		BRIGHT_ENOUGH_LUX = UpperLuxLimit;
	}	


	private boolean AutoTorchMode = false;
    /**
     * Set control of the Torch to Auto or Manual
     */		
    public void setAutoTorchMode(boolean AutoTorchMode) {
		this.AutoTorchMode = AutoTorchMode;
	}			
	
	
    Callback callback;
    public interface  Callback{
        void ambient_light_changed();
//		void brightness_changed();
    }

    private void invokeAmbientLightChanged() {
        if(callback!=null){
            callback.ambient_light_changed();
        }
    }
	
//    private void invokeBrightnessChanged() {
//        if(callback!=null){
//            callback.brightness_changed();
//        }
//    }	


    public void setCallback(Callback callback){
        this.callback = callback;
    }		
	
}
