package main.java.me.dm7.barcodescanner.zxing;

import android.content.Context;
import android.content.res.Configuration;
import android.graphics.Rect;
import android.hardware.Camera;
import android.os.Handler;
import android.os.Looper;
import android.util.AttributeSet;
import android.util.Log;

import com.google.zxing.BarcodeFormat;
import com.google.zxing.BinaryBitmap;
import com.google.zxing.DecodeHintType;
import com.google.zxing.MultiFormatReader;
import com.google.zxing.PlanarYUVLuminanceSource;
import com.google.zxing.LuminanceSource;
import com.google.zxing.ReaderException;
import com.google.zxing.Result;
import com.google.zxing.common.HybridBinarizer;

import java.util.ArrayList;
import java.util.Collection;
import java.util.EnumMap;
import java.util.List;
import java.util.Map;

import anywheresoftware.b4a.BA;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import java.lang.Math;

import main.java.me.dm7.barcodescanner.core.BarcodeScannerView;
import main.java.me.dm7.barcodescanner.core.DisplayUtils;

public class ZXingScannerView extends BarcodeScannerView {
    private static final String TAG = "ZXingScannerView";
    private boolean switcher = true;	

    public interface ResultHandler {
        public void handleResult(Result rawResult);
    }

    private MultiFormatReader mMultiFormatReader;
    public static final List<BarcodeFormat> ALL_FORMATS = new ArrayList<BarcodeFormat>();
    private List<BarcodeFormat> mFormats;
    private ResultHandler mResultHandler;
	public static double currentbrightness = 40.0d;
	private byte[] data1 = null;
	private int avg = 0;
	private int cnt = 0;

    static {
        ALL_FORMATS.add(BarcodeFormat.UPC_A);
        ALL_FORMATS.add(BarcodeFormat.UPC_E);
        ALL_FORMATS.add(BarcodeFormat.EAN_13);
        ALL_FORMATS.add(BarcodeFormat.EAN_8);
        ALL_FORMATS.add(BarcodeFormat.RSS_14);
        ALL_FORMATS.add(BarcodeFormat.CODE_39);
        ALL_FORMATS.add(BarcodeFormat.CODE_93);
        ALL_FORMATS.add(BarcodeFormat.CODE_128);
        ALL_FORMATS.add(BarcodeFormat.ITF);
        ALL_FORMATS.add(BarcodeFormat.CODABAR);
        ALL_FORMATS.add(BarcodeFormat.QR_CODE);
        ALL_FORMATS.add(BarcodeFormat.DATA_MATRIX);
        ALL_FORMATS.add(BarcodeFormat.PDF_417);
    }

    public ZXingScannerView(Context context) {
        super(context);
        initMultiFormatReader();
    }

    public ZXingScannerView(Context context, AttributeSet attributeSet) {
        super(context, attributeSet);
        initMultiFormatReader();
    }

    public void setFormats(List<BarcodeFormat> formats) {
        mFormats = formats;
        initMultiFormatReader();
    }

    public void setResultHandler(ResultHandler resultHandler) {
        mResultHandler = resultHandler;
    }

    public Collection<BarcodeFormat> getFormats() {
        if(mFormats == null) {
            return ALL_FORMATS;
        }
        return mFormats;
    }

    private void initMultiFormatReader() {
        Map<DecodeHintType,Object> hints = new EnumMap<DecodeHintType,Object>(DecodeHintType.class);
        hints.put(DecodeHintType.POSSIBLE_FORMATS, getFormats());
        mMultiFormatReader = new MultiFormatReader();
        mMultiFormatReader.setHints(hints);
    }

    @Override
    public void onPreviewFrame(byte[] data, Camera camera) {
        if(mResultHandler == null) {
            return;
        }
        
        try {
            Camera.Parameters parameters = camera.getParameters();
			
            Camera.Size size = parameters.getPreviewSize();
            int width = size.width;
            int height = size.height;

            if (DisplayUtils.getScreenOrientation(getContext()) == Configuration.ORIENTATION_PORTRAIT) {
                byte[] rotatedData = new byte[data.length];
                for (int y = 0; y < height; y++) {
                    for (int x = 0; x < width; x++)
                        rotatedData[x * height + height - y - 1] = data[x + y * width];
                }
                int tmp = width;
                width = height;
                height = tmp;
                data = rotatedData;
            }
            data1 = data;
			avg = 0;
			cnt = 0;
			for (int i = 0; i < (2 * data.length / 3) - 1; i += 10) {
				avg = avg + (data[i] & 0xFF);
				cnt = cnt + 1;
			}
			avg = avg / cnt;
			
//			BA.Log("width = " + width);
//			BA.Log("height = " + height);
//			BA.Log("data1.length = " + data1.length);
		
            Result rawResult = null;
            PlanarYUVLuminanceSource source = buildLuminanceSource(data, width, height);
			
			
	switcher = !switcher;		
	BinaryBitmap bitmap = null;
	//PlanarYUVLuminanceSource source = CameraManager.get().buildLuminanceSource(data, width, height);
	//if (switcher) {
	//	bitmap = new BinaryBitmap(new HybridBinarizer(source));
	//} else {	
	//    LuminanceSource source1 = source.invert();
    //    bitmap = new BinaryBitmap(new HybridBinarizer(source1));
	//}				
			
			
			
			
			
			
			

            if (source != null) {
                //BinaryBitmap bitmap = new BinaryBitmap(new HybridBinarizer(source));
				
	if (switcher) {
		bitmap = new BinaryBitmap(new HybridBinarizer(source));
	} else {	
	    LuminanceSource source1 = source.invert();
        bitmap = new BinaryBitmap(new HybridBinarizer(source1));
	}								
				
				
				
                try {
                    rawResult = mMultiFormatReader.decodeWithState(bitmap);
                } catch (ReaderException re) {
                    // continue
                } catch (NullPointerException npe) {
                    // This is terrible
                } catch (ArrayIndexOutOfBoundsException aoe) {

                } finally {
                    mMultiFormatReader.reset();
                }
            }

            final Result finalRawResult = rawResult;

            if (finalRawResult != null) {
                Handler handler = new Handler(Looper.getMainLooper());
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        // Stopping the preview can take a little long.
                        // So we want to set result handler to null to discard subsequent calls to
                        // onPreviewFrame.
                        ResultHandler tmpResultHandler = mResultHandler;
                        mResultHandler = null;

                        stopCameraPreview();
                        if (tmpResultHandler != null) {
                            tmpResultHandler.handleResult(finalRawResult);
                        }
                    }
                });
            } else {
                camera.setOneShotPreviewCallback(this);
            }
			

        } catch(RuntimeException e) {
            // TODO: Terrible hack. It is possible that this method is invoked after camera is released.
            //Log.e(TAG, e.toString(), e);
        }
    }

    public void resumeCameraPreview(ResultHandler resultHandler) {
        mResultHandler = resultHandler;
        super.resumeCameraPreview();
    }

    public PlanarYUVLuminanceSource buildLuminanceSource(byte[] data, int width, int height) {
		
		currentbrightness = calculateBrightness(data);
		invokeBrightnessChanged();
		
        Rect rect = getFramingRectInPreview(width, height);
        if (rect == null) {
            return null;
        }
        // Go ahead and assume it's YUV rather than die.
        PlanarYUVLuminanceSource source = null;

        try {
            source = new PlanarYUVLuminanceSource(data, width, height, rect.left, rect.top,
                    rect.width(), rect.height(), false);
        } catch(Exception e) {
        }

        return source;
    }
	
	
	
    private double calculateBrightness(byte[] data) {
		
/* 		for (int j = data.length - 8; j < data.length; j++) {
			BA.Log("data[" + j + "] = " + data[j]);
		} */
		
		double brightness = 0;
		int n = 0;
		int R = 0; int G = 0; int B = 0;
		int nrOfPixels = data.length / 3; // Three bytes per pixel.
		int pixels[] = new int[nrOfPixels];
		for(int i = 0; i < nrOfPixels; i++) {
		   int r = 128 + data[3*i];
		   int g = 128 + data[3*i + 1];
		   int b = 128 + data[3*i + 2];
           n++;
		   brightness = brightness + Math.sqrt((double) (0.299 * r * r) + (double) (0.587 * g * g) + (double) (0.114 * b * b));

		}

		brightness = brightness / n / 255;	
		
	    return brightness;
	}
	
	public int getBrightness() {
		return avg;
	}
	


    Callback callback;
    public interface  Callback{
		void brightness_changed();
    }

	
    private void invokeBrightnessChanged() {
        if(callback!=null){
            callback.brightness_changed();
        }
    }	


    public void setCallback(Callback callback){
        this.callback = callback;
    }			
	
	

	
	
}
